/* Copyright (C) 1999 Chris Vine, G3XXF

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYRIGHT distributed with the source files.

*/

#ifndef KEYBOARD_H
#define KEYBOARD_H

#define CONNECT_SIZE 40
#if CONNECT_SIZE < hisCall_SIZE || CONNECT_SIZE < selCall_SIZE
#error 
#endif

#include <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include <ncurses.h>
// we don't want to use the 'clear' macro in ncurses.h - it will interfere with iostream::clear()
#undef clear
#include <string.h>
#include <ctype.h>
#include "tnc.h"
#include "screen.h"
#include "buffers.h"
#include "prog_defs.h"
#include "pipes.h"

typedef unsigned char uchar;

extern ifstream filein;
extern ofstream fileout;
extern Prog_func prog_func;

void send_parms(WinList&);
void curses_init(void);

class Keyboard_input {
protected:
    int input_confirm(void);
public:
    virtual void action(void) = 0;
    virtual ~Keyboard_input(void) {} // in case a derived class has a destructor
};

class Keyboard_input_with_data: public Keyboard_input {
public:
    enum Sendfile_flag{not_sending, loading_buffer, loading_complete};
protected:
    static MainScreen* mainscreen_ptr;
    static SendWin* sendwin_ptr;
    static ReceiveWin* receivewin_ptr;
    static WinList* screenform_ptr;
    static Pipe_fifo* send_pipe_ptr;
    static Transmit_buffer* tr_buffer_ptr;
    static Keyboard_input* default_input_ptr;
    static Tnc* tnc_ptr;
    static Keyboard_input* keyboard_ptr;
    static int F_KEYmatrix[3][7];
    static int sendfile_flag;
    static int convert_codes_flag;
    static void set_data(MainScreen*, SendWin*, ReceiveWin*, WinList*, Pipe_fifo*,
			   Transmit_buffer*, Tnc*, Keyboard_input*);
    void send_message(char);
    void sendfile(char* filename = 0, int convert_codes = FALSE);
    int is_baudot(char);
public:
    const Keyboard_input* get_keyboard_ptr(void) const {return keyboard_ptr;}
    Sendfile_flag get_sendfile_flag(void) const {return (Sendfile_flag)sendfile_flag;}
    friend void curses_init(void); // this is a friend, because it needs access to F_KEYmatrix[][]
};

class Keyboard_controller;          // forward declaration
class Keyboard_setup_mode;          // forward declaration

class Exit_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    friend Keyboard_setup_mode;
    PopupWin* popupwin_ptr;
    Exit_query(void) {}            // private class - only Keyboard_controller or Keyboard_setup_mode
    void setup(void);              // can create an object
public:   
    void action(void);
};

class Help_file: public Keyboard_input_with_data {
    friend Keyboard_controller;
    Help_file(void) {}            // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Openfile: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Openfile(void) {}            // private class - only Keyboard_controller can create an object
    char filename[256];
    int index;
    void setup(void);
public:
    void action(void);
};

class Input_command: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Input_command(void) {}      // private class - only Keyboard_controller can create an object
    char buffer[256];
    int index;
    void setup(void);
public:
    void action(void);
};

class Callsign: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Callsign(void) {}            // private class - only Keyboard_controller can create an object
    char buffer[hisCall_SIZE + 1];
    int index;
    void setup(void);
public:
    void action(void);
};

class Send_parms_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Send_parms_query(void) {}   // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Connect: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Connect(void) {}            // private class - only Keyboard_controller can create an object
    char buffer[CONNECT_SIZE + 1];
    int index;
    int max_size;
    void setup(void);
public:
    void action(void);
};

class Packet_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Packet_query(void) {}            // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Pactor_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Pactor_query(void) {}            // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Amtor_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Amtor_query(void) {}            // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Ascii_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Ascii_query(void) {}            // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Fec_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Fec_query(void) {}              // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Gmon_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Gmon_query(void) {}             // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Gtor_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Gtor_query(void) {}             // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Lamtor_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Lamtor_query(void) {}           // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Rtty_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Rtty_query(void) {}             // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Tor_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Tor_query(void) {}              // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Disconnect_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    Disconnect_query(void) {}       // private class - only Keyboard_controller can create an object
    void setup(void);
public:
    void action(void);
};

class Send_message_query: public Keyboard_input_with_data {
    friend Keyboard_controller;
    PopupWin* popupwin_ptr;
    char message_num;
    Send_message_query(void) {} // private class - only Keyboard_controller can create an object
    void setup(char);
public:
    void action(void);
};

// This class provides keyboard functions when the Kam is operating in its normal (host) mode

class Keyboard_controller: public Keyboard_input_with_data {
private:
    Exit_query exit_query_obj;
    Help_file help_obj;
    Openfile openfile_obj;
    Input_command input_command_obj;
    Callsign callsign_obj;
    Send_parms_query send_parms_query_obj;
    Connect connect_obj;
    Amtor_query amtor_query_obj;
    Ascii_query ascii_query_obj;
    Fec_query fec_query_obj;
    Gmon_query gmon_query_obj;
    Gtor_query gtor_query_obj;
    Lamtor_query lamtor_query_obj;
    Pactor_query pactor_query_obj;
    Packet_query packet_query_obj;
    Rtty_query rtty_query_obj;
    Tor_query tor_query_obj;
    Disconnect_query disconnect_query_obj;
    Send_message_query send_message_query_obj;
    PopupWin* popupwin_ptr;
public:
    static void send_kamcommand(const char* text, char ascii_stream = -1); // if ascii_stream == -1 
                                                               // the current active stream is used
    static void send_specialcommand(char);
    void action(void);
    Keyboard_controller(MainScreen&, SendWin&, ReceiveWin&, WinList&, Pipe_fifo&,
			  Transmit_buffer&, Tnc&);
};

// This class provides keyboard functions when operating in setup (terminal) mode

class Keyboard_setup_mode: public Keyboard_input_with_data {
private:
    Exit_query exit_query_obj;
public:
    void action(void);
    Keyboard_setup_mode(MainScreen&, SendWin&, ReceiveWin&, WinList&, Pipe_fifo&,
			  Transmit_buffer&, Tnc&);
};


class Gpl_query: public Keyboard_input {
    CopyrightScreen& copyright_screen;
public:
    void action(void);
    Gpl_query(CopyrightScreen& a): copyright_screen(a) {}
};


#endif
