/* Copyright (C) 1999 Chris Vine, G3XXF

This program is distributed under the General Public Licence, version 2.
For particulars of this and relevant disclaimers see the file
COPYRIGHT distributed with the source files.

*/

#ifndef TNC_H
#define TNC_H

#include "prog_defs.h"

typedef unsigned char uchar;

// we can't include screen.h until after class Tnc is defined
// so make some forward declarations

class MainScreen;
class ReceiveWin;
class Transmit_buffer;
class Pipe_fifo;

// the following are commands required by the Kam to change modes and from transmit
// to receive, etc

const char packetCMD = 'X';
const char pactorCMD[] = "PACTOR";
const char rttyCMD[] = "RTTY";
const char asciiCMD[] = "ASCII";
const char amtorCMD[] = "AMTOR";
const char lamtorCMD[] = "LAMTOR";
const char fecCMD[] = "FEC";
const char gtorCMD[] = "GTOR";
const char gmonCMD[] = "GMON";
const char torCMD[] = "TOR";
const char cwCMD[] = "CW";
const char packet_connectCMD[] = "CONNECT ";
const char pactor_connectCMD[] = "PACTOR ";
const char amtor_connectCMD[] = "AMTOR ";
const char gtor_connectCMD[] = "GTOR ";

const char txCMD = 'T';
const char rxCMD = 'E';
const char immediate_rxCMD = 'R';
const char tor_disconnectCMD = 'D';
const char packet_disconnectCMD[] = "DISCONNECT";
const char aborttxCMD = 'A';
const char statusCMD[] = "STATUS";

//const unsigned char FEND = '@';
const unsigned char FEND = 0xc0;
const unsigned char FESC = 0xdb;
const unsigned char TFEND = 0xdc;
const unsigned char TFESC = 0xdd;

////

struct hostframe {
    char command;
    char port;
    char stream;
    char data[513]; // we have no control over the size of received frames; 513 bytes is maximum
    int data_count;
    int FEND_count;
    int bytes_afterFEND;
};

struct Tnc_func {
    enum {packet, pactor, rtty, ascii, amtor,lamtor, fec, gtor, gmon, tor, cw} hfmode;
    enum {tor_pactor_gtor, tor_amtor} tor_connected_mode;
    enum connected_status {disconnected, connected}; connected_status stream_status[MAXUSERS][2];
    char hisCall[MAXUSERS][2][hisCall_SIZE + 1];
    enum {off, on} hisCall_lock, speed_lock;
    enum {no, yes} have_sent_lock;
    char selCall[selCall_SIZE + 1];
    int active_hf_stream;
    int active_vhf_stream;
    int active_port;
    int active_stream(void) const {return active_port ? active_hf_stream : active_vhf_stream;}
    int capturefile_flag;
    struct{int port, stream;} capture_stream;
};

// Note: If Kam port 1 (VHF), is being displayed in the receive window, active_port
// will contain 0, and if Kam port 2 (HF) is being displayed, it will contain 1.

// Active_hf_stream will contain a number representing the hf stream currently
// being displayed, or if a vhf stream is being displayed, the last hf stream
// displayed.  Active_vhf_stream will do the same for the vhf stream.
// The number contained will be 0 for stream A, 1 for stream B, and so on.

// Function int active_stream() is a short-hand way of getting the current active
// stream of the current port being displayed in the receive window.


// The following abstract base class provides virtual methods process_received_byte()
// and send_to_tnc() to make it slightly easier for the user to switch between normal
// mode (running in Kam host mode) and setup mode (running in Kam terminal mode).
// Various other methods are also declared virtual and provide dummy functions in
// order to make it easier to provide a generic means of using the normal mode and
// setup mode derived classes

class Tnc_base {
protected:
    MainScreen* mainscreen_p;           // this win pointer is for process_rx_frame()
    ReceiveWin* receivewin_p;           // ditto
    Transmit_buffer& tr_buffer;
public:
    virtual void process_received_byte(char) = 0;
    virtual void send_to_tnc(Pipe_fifo&, int) = 0;
    virtual int is_freebytes(void) const = 0;
    virtual int get_freebytes(void) const = 0;
    virtual void find_freebytes(void) = 0;
    virtual void get_info(void) = 0;
    void set_wins(MainScreen* a, ReceiveWin* b) {mainscreen_p = a; receivewin_p = b;}
    Tnc_base(Transmit_buffer& a): tr_buffer(a) {}
    virtual ~Tnc_base(void) {} // in case a derived class has a destructor
};

// This class provides the Kam host mode functions when the program is running in 
// normal mode

class Tnc: public Tnc_base {
  // this flag indicates that we have sent the command 'STATUS' to the tnc
  // to check free bytes in the Kam buffer; this flags stops us sending two
  // such requests at the same time
    int check_status_flag;

  // this flag indicates that the frame being/just received is an information
  // (command byte = '?') frame
    int info_received_flag;

    int rx_frame_complete;
    hostframe rx_frame;  // holds frame currently being received
    int kam_freebytes;
    int unsent_frame_flag;
    int tx_frame_index;
    char tx_frame[FRAMEDATA_SIZE + 6];  // include a byte for a final translated FEND or FESC in data part of frame
    int CMDflag;
    char* const tx_data;                // points to start of data in tx_frame
    char CMDchar;
    int de_flag;
    int find_call_index;
    char callbuffer[CALLBUFFER_SIZE + 1];
    void process_rx_frame(void);
    void parse_status(char*);
    void read_state_message(char*, int, int);
    uchar cp437_to_Latin1(uchar);
    uchar Latin1_to_cp437(uchar);
public:
    Tnc_func tnc_func;
    void process_received_byte(char);
    void send_to_tnc(Pipe_fifo&, int);
    int is_freebytes(void) const;
    int get_freebytes(void) const;
    void find_freebytes(void);
    void get_info(void);
    int get_unsent_frame_flag(void) const {return unsent_frame_flag;}
    void find_call(char);
    int is_validcall(const char*);
    void make_selcall(const char*, char*);
    Tnc(Transmit_buffer&);
};

// This class provides Kam terminal mode functions in order to set up the Kam

class Tnc_setup: public Tnc_base {
public:
    void process_received_byte(char);
    void send_to_tnc(Pipe_fifo&, int);
    int is_freebytes(void) const;
    int get_freebytes(void) const;
    void find_freebytes(void);
    void get_info(void);
    Tnc_setup(Transmit_buffer& a): Tnc_base(a) {}
};

#endif
